@@ -1,14 +1,8 @@
 /** @file
   Implement TPM2 Capability related command.
 
-Copyright (c) 2013, Intel Corporation. All rights reserved. <BR>
-This program and the accompanying materials
-are licensed and made available under the terms and conditions of the BSD License
-which accompanies this distribution.  The full text of the license may be found at
-http://opensource.org/licenses/bsd-license.php
-
-THE PROGRAM IS DISTRIBUTED UNDER THE BSD LICENSE ON AN "AS IS" BASIS,
-WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.
+Copyright (c) 2013 - 2018, Intel Corporation. All rights reserved. <BR>
+SPDX-License-Identifier: BSD-2-Clause-Patent
 
 **/
 
@@ -48,25 +42,25 @@ typedef struct {
 /**
   This command returns various information regarding the TPM and its current state.
 
-  The capability parameter determines the category of data returned. The property parameter 
-  selects the first value of the selected category to be returned. If there is no property 
+  The capability parameter determines the category of data returned. The property parameter
+  selects the first value of the selected category to be returned. If there is no property
   that corresponds to the value of property, the next higher value is returned, if it exists.
-  The moreData parameter will have a value of YES if there are more values of the requested 
+  The moreData parameter will have a value of YES if there are more values of the requested
   type that were not returned.
-  If no next capability exists, the TPM will return a zero-length list and moreData will have 
+  If no next capability exists, the TPM will return a zero-length list and moreData will have
   a value of NO.
 
-  NOTE: 
-  To simplify this function, leave returned CapabilityData for caller to unpack since there are 
+  NOTE:
+  To simplify this function, leave returned CapabilityData for caller to unpack since there are
   many capability categories and only few categories will be used in firmware. It means the caller
-  need swap the byte order for the feilds in CapabilityData.
+  need swap the byte order for the fields in CapabilityData.
 
   @param[in]  Capability         Group selection; determines the format of the response.
-  @param[in]  Property           Further definition of information. 
+  @param[in]  Property           Further definition of information.
   @param[in]  PropertyCount      Number of properties of the indicated type to return.
   @param[out] MoreData           Flag to indicate if there are more values of this type.
   @param[out] CapabilityData     The capability data.
-  
+
   @retval EFI_SUCCESS            Operation completed successfully.
   @retval EFI_DEVICE_ERROR       The command was unsuccessful.
 **/
@@ -95,10 +89,10 @@ Tpm2GetCapability (
   SendBuffer.Capability = SwapBytes32 (Capability);
   SendBuffer.Property = SwapBytes32 (Property);
   SendBuffer.PropertyCount = SwapBytes32 (PropertyCount);
- 
+
   SendBufferSize = (UINT32) sizeof (SendBuffer);
   SendBuffer.Header.paramSize = SwapBytes32 (SendBufferSize);
-    
+
   //
   // send Tpm command
   //
@@ -112,15 +106,23 @@ Tpm2GetCapability (
     return EFI_DEVICE_ERROR;
   }
 
+  //
+  // Fail if command failed
+  //
+  if (SwapBytes32(RecvBuffer.Header.responseCode) != TPM_RC_SUCCESS) {
+    DEBUG ((EFI_D_ERROR, "Tpm2GetCapability: Response Code error! 0x%08x\r\n", SwapBytes32(RecvBuffer.Header.responseCode)));
+    return EFI_DEVICE_ERROR;
+  }
+
   //
   // Return the response
   //
   *MoreData = RecvBuffer.MoreData;
   //
-  // Does not unpack all possiable property here, the caller should unpack it and note the byte order.
+  // Does not unpack all possible property here, the caller should unpack it and note the byte order.
   //
   CopyMem (CapabilityData, &RecvBuffer.CapabilityData, RecvBufferSize - sizeof (TPM2_RESPONSE_HEADER) - sizeof (UINT8));
-  
+
   return EFI_SUCCESS;
 }
 
@@ -130,7 +132,7 @@ Tpm2GetCapability (
   This function parse the value got from TPM2_GetCapability and return the Family.
 
   @param[out] Family             The Family of TPM. (a 4-octet character string)
-  
+
   @retval EFI_SUCCESS            Operation completed successfully.
   @retval EFI_DEVICE_ERROR       The command was unsuccessful.
 **/
@@ -142,13 +144,13 @@ Tpm2GetCapabilityFamily (
 {
   TPMS_CAPABILITY_DATA    TpmCap;
   TPMI_YES_NO             MoreData;
-  EFI_STATUS              Status; 
+  EFI_STATUS              Status;
 
   Status = Tpm2GetCapability (
-             TPM_CAP_TPM_PROPERTIES, 
-             TPM_PT_FAMILY_INDICATOR, 
-             1, 
-             &MoreData, 
+             TPM_CAP_TPM_PROPERTIES,
+             TPM_PT_FAMILY_INDICATOR,
+             1,
+             &MoreData,
              &TpmCap
              );
   if (EFI_ERROR (Status)) {
@@ -165,7 +167,7 @@ Tpm2GetCapabilityFamily (
   This function parse the value got from TPM2_GetCapability and return the TPM manufacture ID.
 
   @param[out] ManufactureId      The manufacture ID of TPM.
-  
+
   @retval EFI_SUCCESS            Operation completed successfully.
   @retval EFI_DEVICE_ERROR       The command was unsuccessful.
 **/
@@ -177,19 +179,19 @@ Tpm2GetCapabilityManufactureID (
 {
   TPMS_CAPABILITY_DATA    TpmCap;
   TPMI_YES_NO             MoreData;
-  EFI_STATUS              Status; 
+  EFI_STATUS              Status;
 
   Status = Tpm2GetCapability (
-             TPM_CAP_TPM_PROPERTIES, 
-             TPM_PT_MANUFACTURER, 
-             1, 
-             &MoreData, 
+             TPM_CAP_TPM_PROPERTIES,
+             TPM_PT_MANUFACTURER,
+             1,
+             &MoreData,
              &TpmCap
              );
   if (EFI_ERROR (Status)) {
     return Status;
   }
-  *ManufactureId = SwapBytes32 (TpmCap.data.tpmProperties.tpmProperty->value);
+  *ManufactureId = TpmCap.data.tpmProperties.tpmProperty->value;
 
   return EFI_SUCCESS;
 }
@@ -201,7 +203,7 @@ Tpm2GetCapabilityManufactureID (
 
   @param[out] FirmwareVersion1   The FirmwareVersion1.
   @param[out] FirmwareVersion2   The FirmwareVersion2.
-  
+
   @retval EFI_SUCCESS            Operation completed successfully.
   @retval EFI_DEVICE_ERROR       The command was unsuccessful.
 **/
@@ -214,13 +216,13 @@ Tpm2GetCapabilityFirmwareVersion (
 {
   TPMS_CAPABILITY_DATA    TpmCap;
   TPMI_YES_NO             MoreData;
-  EFI_STATUS              Status; 
+  EFI_STATUS              Status;
 
   Status = Tpm2GetCapability (
-             TPM_CAP_TPM_PROPERTIES, 
-             TPM_PT_FIRMWARE_VERSION_1, 
-             1, 
-             &MoreData, 
+             TPM_CAP_TPM_PROPERTIES,
+             TPM_PT_FIRMWARE_VERSION_1,
+             1,
+             &MoreData,
              &TpmCap
              );
   if (EFI_ERROR (Status)) {
@@ -229,10 +231,10 @@ Tpm2GetCapabilityFirmwareVersion (
   *FirmwareVersion1 = SwapBytes32 (TpmCap.data.tpmProperties.tpmProperty->value);
 
   Status = Tpm2GetCapability (
-             TPM_CAP_TPM_PROPERTIES, 
-             TPM_PT_FIRMWARE_VERSION_2, 
-             1, 
-             &MoreData, 
+             TPM_CAP_TPM_PROPERTIES,
+             TPM_PT_FIRMWARE_VERSION_2,
+             1,
+             &MoreData,
              &TpmCap
              );
   if (EFI_ERROR (Status)) {
@@ -250,7 +252,7 @@ Tpm2GetCapabilityFirmwareVersion (
 
   @param[out] MaxCommandSize     The maximum value for commandSize in a command.
   @param[out] MaxResponseSize    The maximum value for responseSize in a command.
-  
+
   @retval EFI_SUCCESS            Operation completed successfully.
   @retval EFI_DEVICE_ERROR       The command was unsuccessful.
 **/
@@ -266,10 +268,10 @@ Tpm2GetCapabilityMaxCommandResponseSize (
   EFI_STATUS              Status;
 
   Status = Tpm2GetCapability (
-             TPM_CAP_TPM_PROPERTIES, 
-             TPM_PT_MAX_COMMAND_SIZE, 
-             1, 
-             &MoreData, 
+             TPM_CAP_TPM_PROPERTIES,
+             TPM_PT_MAX_COMMAND_SIZE,
+             1,
+             &MoreData,
              &TpmCap
              );
   if (EFI_ERROR (Status)) {
@@ -279,10 +281,10 @@ Tpm2GetCapabilityMaxCommandResponseSize (
   *MaxCommandSize = SwapBytes32 (TpmCap.data.tpmProperties.tpmProperty->value);
 
   Status = Tpm2GetCapability (
-             TPM_CAP_TPM_PROPERTIES, 
-             TPM_PT_MAX_RESPONSE_SIZE, 
-             1, 
-             &MoreData, 
+             TPM_CAP_TPM_PROPERTIES,
+             TPM_PT_MAX_RESPONSE_SIZE,
+             1,
+             &MoreData,
              &TpmCap
              );
   if (EFI_ERROR (Status)) {
@@ -290,17 +292,17 @@ Tpm2GetCapabilityMaxCommandResponseSize (
   }
 
   *MaxResponseSize = SwapBytes32 (TpmCap.data.tpmProperties.tpmProperty->value);
-  return EFI_SUCCESS; 
+  return EFI_SUCCESS;
 }
 
 /**
   This command returns Returns a list of TPMS_ALG_PROPERTIES. Each entry is an
-  algorithm ID and a set of properties of the algorithm. 
+  algorithm ID and a set of properties of the algorithm.
 
   This function parse the value got from TPM2_GetCapability and return the list.
 
   @param[out] AlgList      List of algorithm.
-  
+
   @retval EFI_SUCCESS            Operation completed successfully.
   @retval EFI_DEVICE_ERROR       The command was unsuccessful.
 **/
@@ -314,21 +316,26 @@ Tpm2GetCapabilitySupportedAlg (
   TPMI_YES_NO             MoreData;
   UINTN                   Index;
   EFI_STATUS              Status;
- 
+
   Status = Tpm2GetCapability (
-             TPM_CAP_ALGS, 
-             1, 
-             MAX_CAP_ALGS, 
-             &MoreData, 
+             TPM_CAP_ALGS,
+             1,
+             MAX_CAP_ALGS,
+             &MoreData,
              &TpmCap
              );
   if (EFI_ERROR (Status)) {
     return Status;
   }
-  
+
   CopyMem (AlgList, &TpmCap.data.algorithms, sizeof (TPML_ALG_PROPERTY));
 
   AlgList->count = SwapBytes32 (AlgList->count);
+  if (AlgList->count > MAX_CAP_ALGS) {
+    DEBUG ((DEBUG_ERROR, "Tpm2GetCapabilitySupportedAlg - AlgList->count error %x\n", AlgList->count));
+    return EFI_DEVICE_ERROR;
+  }
+
   for (Index = 0; Index < AlgList->count; Index++) {
     AlgList->algProperties[Index].alg = SwapBytes16 (AlgList->algProperties[Index].alg);
     WriteUnaligned32 ((UINT32 *)&AlgList->algProperties[Index].algProperties, SwapBytes32 (ReadUnaligned32 ((UINT32 *)&AlgList->algProperties[Index].algProperties)));
@@ -343,7 +350,7 @@ Tpm2GetCapabilitySupportedAlg (
   This function parse the value got from TPM2_GetCapability and return the LockoutCounter.
 
   @param[out] LockoutCounter     The LockoutCounter of TPM.
-  
+
   @retval EFI_SUCCESS            Operation completed successfully.
   @retval EFI_DEVICE_ERROR       The command was unsuccessful.
 **/
@@ -355,13 +362,13 @@ Tpm2GetCapabilityLockoutCounter (
 {
   TPMS_CAPABILITY_DATA    TpmCap;
   TPMI_YES_NO             MoreData;
-  EFI_STATUS              Status; 
+  EFI_STATUS              Status;
 
   Status = Tpm2GetCapability (
-             TPM_CAP_TPM_PROPERTIES, 
-             TPM_PT_LOCKOUT_COUNTER, 
-             1, 
-             &MoreData, 
+             TPM_CAP_TPM_PROPERTIES,
+             TPM_PT_LOCKOUT_COUNTER,
+             1,
+             &MoreData,
              &TpmCap
              );
   if (EFI_ERROR (Status)) {
@@ -378,7 +385,7 @@ Tpm2GetCapabilityLockoutCounter (
   This function parse the value got from TPM2_GetCapability and return the LockoutInterval.
 
   @param[out] LockoutInterval    The LockoutInterval of TPM.
-  
+
   @retval EFI_SUCCESS            Operation completed successfully.
   @retval EFI_DEVICE_ERROR       The command was unsuccessful.
 **/
@@ -390,13 +397,13 @@ Tpm2GetCapabilityLockoutInterval (
 {
   TPMS_CAPABILITY_DATA    TpmCap;
   TPMI_YES_NO             MoreData;
-  EFI_STATUS              Status; 
+  EFI_STATUS              Status;
 
   Status = Tpm2GetCapability (
-             TPM_CAP_TPM_PROPERTIES, 
-             TPM_PT_LOCKOUT_INTERVAL, 
-             1, 
-             &MoreData, 
+             TPM_CAP_TPM_PROPERTIES,
+             TPM_PT_LOCKOUT_INTERVAL,
+             1,
+             &MoreData,
              &TpmCap
              );
   if (EFI_ERROR (Status)) {
@@ -414,7 +421,7 @@ Tpm2GetCapabilityLockoutInterval (
 
   @param[out] InputBufferSize    The InputBufferSize of TPM.
                                  the maximum size of a parameter (typically, a TPM2B_MAX_BUFFER)
-  
+
   @retval EFI_SUCCESS            Operation completed successfully.
   @retval EFI_DEVICE_ERROR       The command was unsuccessful.
 **/
@@ -426,13 +433,13 @@ Tpm2GetCapabilityInputBufferSize (
 {
   TPMS_CAPABILITY_DATA    TpmCap;
   TPMI_YES_NO             MoreData;
-  EFI_STATUS              Status; 
+  EFI_STATUS              Status;
 
   Status = Tpm2GetCapability (
-             TPM_CAP_TPM_PROPERTIES, 
-             TPM_PT_INPUT_BUFFER, 
-             1, 
-             &MoreData, 
+             TPM_CAP_TPM_PROPERTIES,
+             TPM_PT_INPUT_BUFFER,
+             1,
+             &MoreData,
              &TpmCap
              );
   if (EFI_ERROR (Status)) {
@@ -449,7 +456,7 @@ Tpm2GetCapabilityInputBufferSize (
   This function parse the value got from TPM2_GetCapability and return the PcrSelection.
 
   @param[out] Pcrs    The Pcr Selection
-  
+
   @retval EFI_SUCCESS            Operation completed successfully.
   @retval EFI_DEVICE_ERROR       The command was unsuccessful.
 **/
@@ -465,10 +472,10 @@ Tpm2GetCapabilityPcrs (
   UINTN                   Index;
 
   Status = Tpm2GetCapability (
-             TPM_CAP_PCRS, 
-             0, 
-             1, 
-             &MoreData, 
+             TPM_CAP_PCRS,
+             0,
+             1,
+             &MoreData,
              &TpmCap
              );
   if (EFI_ERROR (Status)) {
@@ -476,22 +483,123 @@ Tpm2GetCapabilityPcrs (
   }
 
   Pcrs->count = SwapBytes32 (TpmCap.data.assignedPCR.count);
+  if (Pcrs->count > HASH_COUNT) {
+    DEBUG ((DEBUG_ERROR, "Tpm2GetCapabilityPcrs - Pcrs->count error %x\n", Pcrs->count));
+    return EFI_DEVICE_ERROR;
+  }
+
   for (Index = 0; Index < Pcrs->count; Index++) {
     Pcrs->pcrSelections[Index].hash = SwapBytes16 (TpmCap.data.assignedPCR.pcrSelections[Index].hash);
     Pcrs->pcrSelections[Index].sizeofSelect = TpmCap.data.assignedPCR.pcrSelections[Index].sizeofSelect;
+    if (Pcrs->pcrSelections[Index].sizeofSelect > PCR_SELECT_MAX) {
+      DEBUG ((DEBUG_ERROR, "Tpm2GetCapabilityPcrs - sizeofSelect error %x\n", Pcrs->pcrSelections[Index].sizeofSelect));
+      return EFI_DEVICE_ERROR;
+    }
     CopyMem (Pcrs->pcrSelections[Index].pcrSelect, TpmCap.data.assignedPCR.pcrSelections[Index].pcrSelect, Pcrs->pcrSelections[Index].sizeofSelect);
   }
 
   return EFI_SUCCESS;
 }
 
+/**
+  This function will query the TPM to determine which hashing algorithms
+  are supported and which PCR banks are currently active.
+
+  @param[out]  TpmHashAlgorithmBitmap A bitmask containing the algorithms supported by the TPM.
+  @param[out]  ActivePcrBanks         A bitmask containing the PCRs currently allocated.
+
+  @retval     EFI_SUCCESS   TPM was successfully queried and return values can be trusted.
+  @retval     Others        An error occurred, likely in communication with the TPM.
+
+**/
+EFI_STATUS
+EFIAPI
+Tpm2GetCapabilitySupportedAndActivePcrs (
+  OUT UINT32                            *TpmHashAlgorithmBitmap,
+  OUT UINT32                            *ActivePcrBanks
+  )
+{
+  EFI_STATUS            Status;
+  TPML_PCR_SELECTION    Pcrs;
+  UINTN                 Index;
+
+  //
+  // Get supported PCR and current Active PCRs.
+  //
+  Status = Tpm2GetCapabilityPcrs (&Pcrs);
+
+  //
+  // If error, assume that we have at least SHA-1 (and return the error.)
+  //
+  if (EFI_ERROR (Status)) {
+    DEBUG ((EFI_D_ERROR, "GetSupportedAndActivePcrs - Tpm2GetCapabilityPcrs fail!\n"));
+    *TpmHashAlgorithmBitmap = HASH_ALG_SHA1;
+    *ActivePcrBanks         = HASH_ALG_SHA1;
+  }
+  //
+  // Otherwise, process the return data to determine what algorithms are supported
+  // and currently allocated.
+  //
+  else {
+    DEBUG ((EFI_D_INFO, "GetSupportedAndActivePcrs - Count = %08x\n", Pcrs.count));
+    *TpmHashAlgorithmBitmap = 0;
+    *ActivePcrBanks         = 0;
+    for (Index = 0; Index < Pcrs.count; Index++) {
+      switch (Pcrs.pcrSelections[Index].hash) {
+      case TPM_ALG_SHA1:
+        DEBUG ((EFI_D_VERBOSE, "GetSupportedAndActivePcrs - HASH_ALG_SHA1 present.\n"));
+        *TpmHashAlgorithmBitmap |= HASH_ALG_SHA1;
+        if (!IsZeroBuffer (Pcrs.pcrSelections[Index].pcrSelect, Pcrs.pcrSelections[Index].sizeofSelect)) {
+          DEBUG ((EFI_D_VERBOSE, "GetSupportedAndActivePcrs - HASH_ALG_SHA1 active.\n"));
+          *ActivePcrBanks |= HASH_ALG_SHA1;
+        }
+        break;
+      case TPM_ALG_SHA256:
+        DEBUG ((EFI_D_VERBOSE, "GetSupportedAndActivePcrs - HASH_ALG_SHA256 present.\n"));
+        *TpmHashAlgorithmBitmap |= HASH_ALG_SHA256;
+        if (!IsZeroBuffer (Pcrs.pcrSelections[Index].pcrSelect, Pcrs.pcrSelections[Index].sizeofSelect)) {
+          DEBUG ((EFI_D_VERBOSE, "GetSupportedAndActivePcrs - HASH_ALG_SHA256 active.\n"));
+          *ActivePcrBanks |= HASH_ALG_SHA256;
+        }
+        break;
+      case TPM_ALG_SHA384:
+        DEBUG ((EFI_D_VERBOSE, "GetSupportedAndActivePcrs - HASH_ALG_SHA384 present.\n"));
+        *TpmHashAlgorithmBitmap |= HASH_ALG_SHA384;
+        if (!IsZeroBuffer (Pcrs.pcrSelections[Index].pcrSelect, Pcrs.pcrSelections[Index].sizeofSelect)) {
+          DEBUG ((EFI_D_VERBOSE, "GetSupportedAndActivePcrs - HASH_ALG_SHA384 active.\n"));
+          *ActivePcrBanks |= HASH_ALG_SHA384;
+        }
+        break;
+      case TPM_ALG_SHA512:
+        DEBUG ((EFI_D_VERBOSE, "GetSupportedAndActivePcrs - HASH_ALG_SHA512 present.\n"));
+        *TpmHashAlgorithmBitmap |= HASH_ALG_SHA512;
+        if (!IsZeroBuffer (Pcrs.pcrSelections[Index].pcrSelect, Pcrs.pcrSelections[Index].sizeofSelect)) {
+          DEBUG ((EFI_D_VERBOSE, "GetSupportedAndActivePcrs - HASH_ALG_SHA512 active.\n"));
+          *ActivePcrBanks |= HASH_ALG_SHA512;
+        }
+        break;
+      case TPM_ALG_SM3_256:
+        DEBUG ((EFI_D_VERBOSE, "GetSupportedAndActivePcrs - HASH_ALG_SM3_256 present.\n"));
+        *TpmHashAlgorithmBitmap |= HASH_ALG_SM3_256;
+        if (!IsZeroBuffer (Pcrs.pcrSelections[Index].pcrSelect, Pcrs.pcrSelections[Index].sizeofSelect)) {
+          DEBUG ((EFI_D_VERBOSE, "GetSupportedAndActivePcrs - HASH_ALG_SM3_256 active.\n"));
+          *ActivePcrBanks |= HASH_ALG_SM3_256;
+        }
+        break;
+      }
+    }
+  }
+
+  return Status;
+}
+
 /**
   This command returns the information of TPM AlgorithmSet.
 
   This function parse the value got from TPM2_GetCapability and return the AlgorithmSet.
 
   @param[out] AlgorithmSet    The AlgorithmSet of TPM.
-  
+
   @retval EFI_SUCCESS            Operation completed successfully.
   @retval EFI_DEVICE_ERROR       The command was unsuccessful.
 **/
@@ -503,13 +611,13 @@ Tpm2GetCapabilityAlgorithmSet (
 {
   TPMS_CAPABILITY_DATA    TpmCap;
   TPMI_YES_NO             MoreData;
-  EFI_STATUS              Status; 
+  EFI_STATUS              Status;
 
   Status = Tpm2GetCapability (
-             TPM_CAP_TPM_PROPERTIES, 
-             TPM_PT_ALGORITHM_SET, 
-             1, 
-             &MoreData, 
+             TPM_CAP_TPM_PROPERTIES,
+             TPM_PT_ALGORITHM_SET,
+             1,
+             &MoreData,
              &TpmCap
              );
   if (EFI_ERROR (Status)) {
